@@ -1,7 +1,5 @@
 -- | This module provides a type and functions for handling focus rings
--- of widgets. Note that this interface is merely provided for managing
--- the focus state for a sequence of resource names; it does not do
--- anything beyond keep track of that.
+-- of values.
 --
 -- This interface is experimental.
 module Brick.Focus
@@ -10,6 +8,7 @@ module Brick.Focus
   , focusNext
   , focusPrev
   , focusGetCurrent
+  , focusSetCurrent
   , focusRingCursor
   , withFocusRing
   , focusRingModify
@@ -31,13 +30,13 @@ newtype FocusRing n = FocusRing (C.CList n)
 focusRing :: [n] -> FocusRing n
 focusRing = FocusRing . C.fromList
 
--- | Advance focus to the next widget in the ring.
+-- | Advance focus to the next value in the ring.
 focusNext :: FocusRing n -> FocusRing n
 focusNext r@(FocusRing l)
     | C.isEmpty l = r
     | otherwise = FocusRing $ C.rotR l
 
--- | Advance focus to the previous widget in the ring.
+-- | Advance focus to the previous value in the ring.
 focusPrev :: FocusRing n -> FocusRing n
 focusPrev r@(FocusRing l)
     | C.isEmpty l = r
@@ -69,6 +68,14 @@ withFocusRing ring f a = f (focusGetCurrent ring == Just (getName a)) a
 focusGetCurrent :: FocusRing n -> Maybe n
 focusGetCurrent (FocusRing l) = C.focus l
 
+-- | Set the currently-focused resource name in the ring, provided the
+-- name is in the ring. Otherwise return the ring unmodified.
+focusSetCurrent :: (Eq n) => n -> FocusRing n -> FocusRing n
+focusSetCurrent n r@(FocusRing l) =
+    case C.rotateTo n l of
+        Nothing -> r
+        Just l' -> FocusRing l'
+
 -- | Modify the internal circular list structure of a focus ring
 -- directly. This function permits modification of the circular list
 -- using the rich Data.CircularList API.
