@@ -4,6 +4,7 @@ module Brick.Main
   , customMain
   , simpleMain
   , resizeOrQuit
+  , simpleApp
 
   -- * Event handler functions
   , continue
@@ -41,7 +42,7 @@ module Brick.Main
 where
 
 import Control.Exception (finally)
-import Lens.Micro ((^.), (&), (.~))
+import Lens.Micro ((^.), (&), (.~), (%~), _1, _2)
 import Control.Monad (forever)
 import Control.Monad.Trans.Class (lift)
 import Control.Monad.Trans.State
@@ -129,14 +130,24 @@ simpleMain :: (Ord n)
            => Widget n
            -- ^ The widget to draw.
            -> IO ()
-simpleMain w =
-    let app = App { appDraw = const [w]
-                  , appHandleEvent = resizeOrQuit
-                  , appStartEvent = return
-                  , appAttrMap = const $ attrMap defAttr []
-                  , appChooseCursor = neverShowCursor
-                  }
-    in defaultMain app ()
+simpleMain w = defaultMain (simpleApp w) ()
+
+-- | A simple application with reasonable defaults to be overridden as
+-- desired:
+--
+-- * Draws only the specified widget
+-- * Quits on any event other than resizes
+-- * Has no start event handler
+-- * Provides no attribute map
+-- * Never shows any cursors
+simpleApp :: Widget n -> App s e n
+simpleApp w =
+    App { appDraw = const [w]
+        , appHandleEvent = resizeOrQuit
+        , appStartEvent = return
+        , appAttrMap = const $ attrMap defAttr []
+        , appChooseCursor = neverShowCursor
+        }
 
 -- | An event-handling function which continues execution of the event
 -- loop only when resize events occur; all other types of events trigger
@@ -154,7 +165,7 @@ readBrickEvent :: BChan (BrickEvent n e) -> BChan e -> IO (BrickEvent n e)
 readBrickEvent brickChan userChan = either id AppEvent <$> readBChan2 brickChan userChan
 
 runWithNewVty :: (Ord n)
-              => IO Vty
+              => Vty
               -> BChan (BrickEvent n e)
               -> Maybe (BChan e)
               -> App s e n
@@ -198,20 +209,22 @@ customMain :: (Ord n)
            -- ^ The initial application state.
            -> IO s
 customMain buildVty mUserChan app initialAppState = do
-    let run rs st brickChan = do
-            result <- runWithNewVty buildVty brickChan mUserChan app rs st
+    initialVty <- buildVty
+    let run vty rs st brickChan = do
+            result <- runWithNewVty vty brickChan mUserChan app rs st
             case result of
                 InternalHalt s -> return s
                 InternalSuspendAndResume newRS action -> do
                     newAppState <- action
-                    run newRS newAppState brickChan
+                    newVty <- buildVty
+                    run newVty newRS newAppState brickChan
 
         emptyES = ES [] []
-        eventRO = EventRO M.empty Nothing mempty
+        eventRO = EventRO M.empty initialVty mempty
     (st, eState) <- runStateT (runReaderT (runEventM (appStartEvent app initialAppState)) eventRO) emptyES
     let initialRS = RS M.empty (esScrollRequests eState) S.empty mempty []
     brickChan <- newBChan 20
-    run initialRS st brickChan
+    run initialVty initialRS st brickChan
 
 supplyVtyEvents :: Vty -> BChan (BrickEvent n e) -> IO ()
 supplyVtyEvents vty chan =
@@ -250,7 +263,17 @@ runVty vty readEvent app appState rs = do
                             let localCoords = Location (lc, lr)
                                 lc = c - ec + oC
                                 lr = r - er + oR
-                            return (MouseDown n button mods localCoords, firstRS, exts)
+
+                                -- If the clicked extent was a viewport,
+                                -- adjust the local coordinates by
+                                -- adding the viewport upper-left corner
+                                -- offset.
+                                newCoords = case M.lookup n (viewportMap firstRS) of
+                                  Nothing -> localCoords
+                                  Just vp -> localCoords & _1 %~ (+ (vp^.vpLeft))
+                                                         & _2 %~ (+ (vp^.vpTop))
+
+                            return (MouseDown n button mods newCoords, firstRS, exts)
                         False -> return (e, firstRS, exts)
                 _ -> return (e, firstRS, exts)
         VtyEvent (EvMouseUp c r button) -> do
@@ -271,7 +294,7 @@ runVty vty readEvent app appState rs = do
         _ -> return (e, firstRS, exts)
 
     let emptyES = ES [] []
-        eventRO = EventRO (viewportMap nextRS) (Just vty) nextExts
+        eventRO = EventRO (viewportMap nextRS) vty nextExts
 
     (next, eState) <- runStateT (runReaderT (runEventM (appHandleEvent app appState e'))
                                 eventRO) emptyES
@@ -320,7 +343,7 @@ findClickedExtents_ :: (Int, Int) -> [Extent n] -> [Extent n]
 findClickedExtents_ pos = reverse . filter (clickedExtent pos)
 
 -- | Get the Vty handle currently in use.
-getVtyHandle :: EventM n (Maybe Vty)
+getVtyHandle :: EventM n Vty
 getVtyHandle = EventM $ asks eventVtyHandle
 
 -- | Invalidate the rendering cache entry with the specified resource
@@ -334,9 +357,8 @@ invalidateCache :: EventM n ()
 invalidateCache = EventM $ do
     lift $ modify (\s -> s { cacheInvalidateRequests = InvalidateEntire : cacheInvalidateRequests s })
 
-withVty :: IO Vty -> (Vty -> IO a) -> IO a
-withVty buildVty useVty = do
-    vty <- buildVty
+withVty :: Vty -> (Vty -> IO a) -> IO a
+withVty vty useVty = do
     useVty vty `finally` shutdown vty
 
 renderApp :: Vty -> App s e n -> s -> RenderState n -> IO (RenderState n, [Extent n])
