@@ -18,20 +18,11 @@ module Brick.Widgets.Border
   -- * Drawing single border elements
   , borderElem
 
-  -- * Border attribute names
+  -- * Attribute names
   , borderAttr
-  , vBorderAttr
-  , hBorderAttr
-  , hBorderLabelAttr
-  , tlCornerAttr
-  , trCornerAttr
-  , blCornerAttr
-  , brCornerAttr
   )
 where
 
-import Data.Monoid ((<>))
-
 #if !MIN_VERSION_base(4,8,0)
 import Control.Applicative ((<$>))
 #endif
@@ -49,34 +40,6 @@ import Brick.Widgets.Border.Style (BorderStyle(..))
 borderAttr :: AttrName
 borderAttr = "border"
 
--- | The vertical border attribute name.
-vBorderAttr :: AttrName
-vBorderAttr = borderAttr <> "vertical"
-
--- | The horizontal border attribute name.
-hBorderAttr :: AttrName
-hBorderAttr = borderAttr <> "horizontal"
-
--- | The attribute used for horizontal border labels.
-hBorderLabelAttr :: AttrName
-hBorderLabelAttr = hBorderAttr <> "label"
-
--- | The attribute used for border box top-left corners.
-tlCornerAttr :: AttrName
-tlCornerAttr = borderAttr <> "corner" <> "tl"
-
--- | The attribute used for border box top-right corners.
-trCornerAttr :: AttrName
-trCornerAttr = borderAttr <> "corner" <> "tr"
-
--- | The attribute used for border box bottom-left corners.
-blCornerAttr :: AttrName
-blCornerAttr = borderAttr <> "corner" <> "bl"
-
--- | The attribute used for border box bottom-right corners.
-brCornerAttr :: AttrName
-brCornerAttr = borderAttr <> "corner" <> "br"
-
 -- | Draw the specified border element using the active border style
 -- using 'borderAttr'.
 borderElem :: (BorderStyle -> Char) -> Widget n
@@ -91,6 +54,13 @@ border = border_ Nothing
 
 -- | Put a border around the specified widget with the specified label
 -- widget placed in the middle of the top horizontal border.
+--
+-- Note that a border will wrap its child widget as tightly as possible,
+-- which means that if the child widget is narrower than the label
+-- widget, the label widget will be truncated. If you want to avoid
+-- this behavior, add a 'fill' or other space-filling wrapper to the
+-- bordered widget so that it takes up enough room to make the border
+-- horizontally able to avoid truncating the label.
 borderWithLabel :: Widget n
                 -- ^ The label widget
                 -> Widget n
@@ -101,19 +71,14 @@ borderWithLabel label = border_ (Just label)
 border_ :: Maybe (Widget n) -> Widget n -> Widget n
 border_ label wrapped =
     Widget (hSize wrapped) (vSize wrapped) $ do
-      bs <- ctxBorderStyle <$> getContext
       c <- getContext
 
       middleResult <- render $ hLimit (c^.availWidthL - 2)
                              $ vLimit (c^.availHeightL - 2)
                              $ wrapped
 
-      let top = (withAttr tlCornerAttr $ str [bsCornerTL bs])
-                <+> hBorder_ label <+>
-                (withAttr trCornerAttr $ str [bsCornerTR bs])
-          bottom = (withAttr blCornerAttr $ str [bsCornerBL bs])
-                   <+> hBorder <+>
-                   (withAttr brCornerAttr $ str [bsCornerBR bs])
+      let top = (borderElem bsCornerTL) <+> hBorder_ label <+> (borderElem bsCornerTR)
+          bottom = (borderElem bsCornerBL) <+> hBorder <+> (borderElem bsCornerBR)
           middle = vBorder <+> (Widget Fixed Fixed $ return middleResult) <+> vBorder
           total = top <=> middle <=> bottom
 
@@ -136,12 +101,12 @@ hBorder_ :: Maybe (Widget n) -> Widget n
 hBorder_ label =
     Widget Greedy Fixed $ do
       bs <- ctxBorderStyle <$> getContext
-      let msg = maybe (str [bsHorizontal bs]) (withAttr hBorderLabelAttr) label
-      render $ vLimit 1 $ withAttr hBorderAttr $ hCenterWith (Just $ bsHorizontal bs) msg
+      let msg = maybe (str [bsHorizontal bs]) id label
+      render $ vLimit 1 $ withAttr borderAttr $ hCenterWith (Just $ bsHorizontal bs) msg
 
 -- | A vertical border.  Fills all vertical space.
 vBorder :: Widget n
 vBorder =
     Widget Fixed Greedy $ do
       bs <- ctxBorderStyle <$> getContext
-      render $ hLimit 1 $ withAttr vBorderAttr $ fill (bsVertical bs)
+      render $ hLimit 1 $ withAttr borderAttr $ fill (bsVertical bs)
