@@ -2,6 +2,7 @@
 {-# LANGUAGE TupleSections #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE OverloadedStrings #-}
 -- | This module provides the core widget combinators and rendering
 -- routines. Everything this library does is in terms of these basic
 -- primitives.
@@ -11,8 +12,13 @@ module Brick.Widgets.Core
   , emptyWidget
   , raw
   , txt
+  , txtWrap
+  , txtWrapWith
   , str
+  , strWrap
+  , strWrapWith
   , fill
+  , hyperlink
 
   -- * Padding
   , padLeft
@@ -32,9 +38,11 @@ module Brick.Widgets.Core
   -- * Limits
   , hLimit
   , vLimit
+  , setAvailableSize
 
   -- * Attribute management
   , withDefAttr
+  , modifyDefAttr
   , withAttr
   , forceAttr
   , overrideAttr
@@ -84,7 +92,7 @@ import Control.Applicative
 import Data.Monoid ((<>), mempty)
 #endif
 
-import Lens.Micro ((^.), (.~), (&), (%~), to, _1, _2, each, to, ix, Lens')
+import Lens.Micro ((^.), (.~), (&), (%~), to, _1, _2, each, to, Lens')
 import Lens.Micro.Mtl (use, (%=))
 import Control.Monad ((>=>),when)
 import Control.Monad.Trans.State.Lazy
@@ -100,6 +108,8 @@ import Data.List (sortBy, partition)
 import qualified Graphics.Vty as V
 import Control.DeepSeq
 
+import Text.Wrap (wrapTextToLines, WrapSettings, defaultWrapSettings)
+
 import Brick.Types
 import Brick.Types.Internal
 import Brick.Widgets.Border.Style
@@ -195,13 +205,56 @@ takeColumns numCols (c:cs) =
             then c : takeColumns (numCols - w) cs
             else ""
 
+-- | Make a widget from a string, but wrap the words in the input's
+-- lines at the available width using the default wrapping settings.
+strWrap :: String -> Widget n
+strWrap = strWrapWith defaultWrapSettings
+
+-- | Make a widget from a string, but wrap the words in the input's
+-- lines at the available width using the specified wrapping settings.
+strWrapWith :: WrapSettings -> String -> Widget n
+strWrapWith settings t = txtWrapWith settings $ T.pack t
+
+safeTextWidth :: T.Text -> Int
+safeTextWidth = V.safeWcswidth . T.unpack
+
+-- | Make a widget from text, but wrap the words in the input's lines at
+-- the available width using the default wrapping settings.
+txtWrap :: T.Text -> Widget n
+txtWrap = txtWrapWith defaultWrapSettings
+
+-- | Make a widget from text, but wrap the words in the input's lines at
+-- the available width using the specified wrapping settings.
+txtWrapWith :: WrapSettings -> T.Text -> Widget n
+txtWrapWith settings s =
+    Widget Greedy Fixed $ do
+      c <- getContext
+      let theLines = fixEmpty <$> wrapTextToLines settings (c^.availWidthL) s
+          fixEmpty l | T.null l = " "
+                     | otherwise = l
+      case force theLines of
+          [] -> return emptyResult
+          [one] -> return $ emptyResult & imageL .~ (V.text' (c^.attrL) one)
+          multiple ->
+              let maxLength = maximum $ safeTextWidth <$> multiple
+                  lineImgs = lineImg <$> multiple
+                  lineImg lStr = V.text' (c^.attrL)
+                                   (lStr <> T.replicate (maxLength - safeTextWidth lStr) " ")
+              in return $ emptyResult & imageL .~ (V.vertCat lineImgs)
+
 -- | Build a widget from a 'String'. Breaks newlines up and space-pads
 -- short lines out to the length of the longest line.
+--
+-- The input string must not contain tab characters. If it does,
+-- interface corruption will result since the terminal will likely
+-- render it as taking up more than a single column. The caller should
+-- replace tabs with the appropriate number of spaces as desired.
 str :: String -> Widget n
 str s =
     Widget Fixed Fixed $ do
       c <- getContext
       let theLines = fixEmpty <$> (dropUnused . lines) s
+          fixEmpty :: String -> String
           fixEmpty [] = " "
           fixEmpty l = l
           dropUnused l = takeColumns (availWidth c) <$> take (availHeight c) l
@@ -214,11 +267,26 @@ str s =
                   lineImg lStr = V.string (c^.attrL) (lStr ++ replicate (maxLength - V.safeWcswidth lStr) ' ')
               in return $ emptyResult & imageL .~ (V.vertCat lineImgs)
 
--- | Build a widget from a one-line 'T.Text' value. Behaves the same as
--- 'str'.
+-- | Build a widget from a 'T.Text' value. Behaves the same as 'str'
+-- when the input contains multiple lines.
+--
+-- The input string must not contain tab characters. If it does,
+-- interface corruption will result since the terminal will likely
+-- render it as taking up more than a single column. The caller should
+-- replace tabs with the appropriate number of spaces as desired.
 txt :: T.Text -> Widget n
 txt = str . T.unpack
 
+-- | Hyperlink the given widget to the specified URL. Not all terminal
+-- emulators support this. In those that don't, this should have no
+-- discernible effect.
+hyperlink :: T.Text -> Widget n -> Widget n
+hyperlink url p =
+    Widget (hSize p) (vSize p) $ do
+        c <- getContext
+        let attr = attrMapLookup (c^.ctxAttrNameL) (c^.ctxAttrMapL) `V.withURL` url
+        withReaderT (& ctxAttrMapL %~ setDefaultAttr attr) (render p)
+
 -- | Pad the specified widget on the left. If max padding is used, this
 -- grows greedily horizontally; otherwise it defers to the padded
 -- widget.
@@ -455,9 +523,10 @@ renderBox br ws =
               let remainingPrimary = c^.(contextPrimary br) -
                                      (sum $ (^._2.imageL.(to $ imagePrimary br)) <$> renderedHis)
                   primaryPerLow = remainingPrimary `div` length ls
-                  padFirst = remainingPrimary - (primaryPerLow * length ls)
+                  rest = remainingPrimary - (primaryPerLow * length ls)
                   secondaryPerLow = c^.(contextSecondary br)
-                  primaries = replicate (length ls) primaryPerLow & ix 0 %~ (+ padFirst)
+                  primaries = replicate rest (primaryPerLow + 1) <>
+                              replicate (length ls - rest) primaryPerLow
 
               let renderLow ((i, prim), pri) =
                       (i,) <$> (render $ limitPrimary br pri
@@ -496,7 +565,7 @@ renderBox br ws =
 hLimit :: Int -> Widget n -> Widget n
 hLimit w p =
     Widget Fixed (vSize p) $
-      withReaderT (& availWidthL .~ w) $ render $ cropToContext p
+      withReaderT (& availWidthL %~ (min w)) $ render $ cropToContext p
 
 -- | Limit the space available to the specified widget to the specified
 -- number of rows. This is important for constraining the vertical
@@ -505,7 +574,16 @@ hLimit w p =
 vLimit :: Int -> Widget n -> Widget n
 vLimit h p =
     Widget (hSize p) Fixed $
-      withReaderT (& availHeightL .~ h) $ render $ cropToContext p
+      withReaderT (& availHeightL %~ (min h)) $ render $ cropToContext p
+
+-- | Set the rendering context height and width for this widget. This
+-- is useful for relaxing the rendering size constraints on e.g. layer
+-- widgets where cropping to the screen size is undesirable.
+setAvailableSize :: (Int, Int) -> Widget n -> Widget n
+setAvailableSize (w, h) p =
+    Widget Fixed Fixed $
+      withReaderT (\c -> c & availHeightL .~ h & availWidthL .~ w) $
+        render $ cropToContext p
 
 -- | When drawing the specified widget, set the current attribute used
 -- for drawing to the one with the specified name. Note that the widget
@@ -519,6 +597,16 @@ withAttr an p =
     Widget (hSize p) (vSize p) $
       withReaderT (& ctxAttrNameL .~ an) (render p)
 
+-- | Update the attribute map while rendering the specified widget: set
+-- its new default attribute to the one that we get by looking up the
+-- specified attribute name in the map and then modifying it with the
+-- specified function.
+modifyDefAttr :: (V.Attr -> V.Attr) -> Widget n -> Widget n
+modifyDefAttr f p =
+    Widget (hSize p) (vSize p) $ do
+        c <- getContext
+        withReaderT (& ctxAttrMapL %~ (setDefaultAttr (f $ getDefaultAttr (c^.ctxAttrMapL)))) (render p)
+
 -- | Update the attribute map while rendering the specified widget: set
 -- its new default attribute to the one that we get by looking up the
 -- specified attribute name in the map.
@@ -526,7 +614,7 @@ withDefAttr :: AttrName -> Widget n -> Widget n
 withDefAttr an p =
     Widget (hSize p) (vSize p) $ do
         c <- getContext
-        withReaderT (& ctxAttrMapL %~ (setDefault (attrMapLookup an (c^.ctxAttrMapL)))) (render p)
+        withReaderT (& ctxAttrMapL %~ (setDefaultAttr (attrMapLookup an (c^.ctxAttrMapL)))) (render p)
 
 -- | When rendering the specified widget, update the attribute map with
 -- the specified transformation.
@@ -678,7 +766,7 @@ viewport :: (Ord n, Show n)
          -- ^ The widget to be rendered in the scrollable viewport
          -> Widget n
 viewport vpname typ p =
-    Widget Greedy Greedy $ do
+    clickable vpname $ Widget Greedy Greedy $ do
       -- First, update the viewport size.
       c <- getContext
       let newVp = VP 0 0 newSize
