@@ -19,6 +19,7 @@ module Brick.Widgets.List
 
   -- * Handling events
   , handleListEvent
+  , handleListEventVi
 
   -- * Lenses
   , listElementsL
@@ -31,6 +32,9 @@ module Brick.Widgets.List
   , listMoveTo
   , listMoveUp
   , listMoveDown
+  , listMoveByPages
+  , listMovePageUp
+  , listMovePageDown
   , listInsert
   , listRemove
   , listReplace
@@ -55,7 +59,7 @@ import Data.Traversable (Traversable)
 import Lens.Micro ((^.), (&), (.~), (%~), _2)
 import Data.Maybe (fromMaybe)
 import Data.Monoid ((<>))
-import Graphics.Vty (Event(..), Key(..))
+import Graphics.Vty (Event(..), Key(..), Modifier(..))
 import qualified Data.Vector as V
 
 import Brick.Types
@@ -91,18 +95,42 @@ handleListEvent e theList =
         EvKey KDown [] -> return $ listMoveDown theList
         EvKey KHome [] -> return $ listMoveTo 0 theList
         EvKey KEnd [] -> return $ listMoveTo (V.length $ listElements theList) theList
-        EvKey KPageDown [] -> do
-            v <- lookupViewport (theList^.listNameL)
-            case v of
-                Nothing -> return theList
-                Just vp -> return $ listMoveBy (vp^.vpSize._2 `div` theList^.listItemHeightL) theList
-        EvKey KPageUp [] -> do
-            v <- lookupViewport (theList^.listNameL)
-            case v of
-                Nothing -> return theList
-                Just vp -> return $ listMoveBy (negate $ vp^.vpSize._2 `div` theList^.listItemHeightL) theList
+        EvKey KPageDown [] -> listMovePageDown theList
+        EvKey KPageUp [] -> listMovePageUp theList
         _ -> return theList
 
+-- | Enable list movement with the vi keys with a fallback if none
+-- match. Use (handleListEventVi handleListEvent) in place of
+-- handleListEvent to add the vi keys bindings to the standard ones.
+-- Movements handled include:
+--
+-- * Up             (k)
+-- * Down           (j)
+-- * Page Up        (Ctrl-b)
+-- * Page Down      (Ctrl-f)
+-- * Half Page Up   (Ctrl-u)
+-- * Half Page Down (Ctrl-d)
+-- * Top            (g)
+-- * Bottom         (G)
+handleListEventVi :: (Ord n)
+                  => (Event -> List n e -> EventM n (List n e))
+                  -- ^ Fallback event handler to use if none of the vi keys
+                  -- match.
+                  -> Event
+                  -> List n e
+                  -> EventM n (List n e)
+handleListEventVi fallback e theList =
+    case e of
+        EvKey (KChar 'k') [] -> return $ listMoveUp theList
+        EvKey (KChar 'j') [] -> return $ listMoveDown theList
+        EvKey (KChar 'g') [] -> return $ listMoveTo 0 theList
+        EvKey (KChar 'G') [] -> return $ listMoveTo (V.length $ listElements theList) theList
+        EvKey (KChar 'f') [MCtrl] -> listMovePageDown theList
+        EvKey (KChar 'b') [MCtrl] -> listMovePageUp theList
+        EvKey (KChar 'd') [MCtrl] -> listMoveByPages 0.5 theList
+        EvKey (KChar 'u') [MCtrl] -> listMoveByPages (-0.5) theList
+        _ -> fallback e theList
+
 -- | The top-level attribute used for the entire list.
 listAttr :: AttrName
 listAttr = "list"
@@ -244,11 +272,30 @@ listReplace es idx l =
 listMoveUp :: List n e -> List n e
 listMoveUp = listMoveBy (-1)
 
+-- | Move the list selected index up by one page.
+listMovePageUp :: (Ord n) => List n e -> EventM n (List n e)
+listMovePageUp theList = listMoveByPages (-1) theList
+
 -- | Move the list selected index down by one. (Moves the cursor down,
 -- adds one to the index.)
 listMoveDown :: List n e -> List n e
 listMoveDown = listMoveBy 1
 
+-- | Move the list selected index down by one page.
+listMovePageDown :: (Ord n) => List n e -> EventM n (List n e)
+listMovePageDown theList = listMoveByPages 1 theList
+
+-- | Move the list selected index by some (fractional) number of pages.
+listMoveByPages :: (Ord n, RealFrac m) => m -> List n e -> EventM n (List n e)
+listMoveByPages pages theList = do
+    v <- lookupViewport (theList^.listNameL)
+    case v of
+        Nothing -> return theList
+        Just vp -> let
+            nElems = round $ pages * (fromIntegral $ vp^.vpSize._2) / (fromIntegral $ theList^.listItemHeightL)
+          in
+            return $ listMoveBy nElems theList
+
 -- | Move the list selected index by the specified amount, subject to
 -- validation.
 listMoveBy :: Int -> List n e -> List n e
